/* 
 * Author: Bartosz Jankowski - T-TIZ
 * bjankows@stud.elka.pw.edu.pl
 * biblioteka stl, opis algorytmu : set_difference
 * File: set_difference.cpp
 */

/* 
 * Algorytm set_difference biblioteki STL znjaduje si w komponencie algorytm.
 * Aby zaprezentowac jego dzialanie wyobrazmy sobie prosta mini gre:
 * Mamy 2 postaci: gracza i straznika, kazda z postaci ma swoje przedmioty/ekwipunek.
 * Straznik nie przepusci gracza dopoki ten nie bedzie mial wszystkich wymaganych przedmiotow.
 * Do sprawdzenia czy gracz ma juz wszystkie 'magiczne' przedmioty wykorzystamy set_difference.
 */
#include <iostream>
#include <string>
#include <vector>

/*
 * Tu znajduje sie m.in set_difference
 */
#include <algorithm>

using namespace std;

/*
 * Klasa reprezentuje postac w grze
 */
class GameCharacter				
{
public:

	/*
	 * Przedmioty ktore postac ma w ekwipunku. (vector->see STL reference)
	 */
	vector<string> items;
	GameCharacter(){}
	~GameCharacter(){}

	/*
	 * Metoda dodaje przedmiot do ekwipunku postaci
	 */
	void addItem(string item)
	{
		items.push_back(item);
	}

};

int main () {
//***************INICJALIZUJEMY EKWIPUNKI POSTACI****************
GameCharacter GateKeeper;										//Postac w grze Straznik Bramy
GateKeeper.addItem("Dragon Key");								//
GateKeeper.addItem("Silver Key");								// Przedmioty, ktre naley posiada,
GateKeeper.addItem("Red Orb");									// aby stranik otworzy brame.
GateKeeper.addItem("Vampire Fang");								//

GameCharacter Player;											//Postac w grze gracz
Player.addItem("Dragon Key");									//
Player.addItem("Bannana");										//
Player.addItem("Wine");											// 
Player.addItem("Red Orb");										//Ekwipunek gracza
Player.addItem("Helmet");										//
Player.addItem("Shield");										//
Player.addItem("Sword");										//

vector<string> missingItems(GateKeeper.items.size());			//Vector przedmiotw ktrych gracz nie posiada do przejscia przez brame 
vector<string>::iterator b = missingItems.begin();				//iterator na pierwszy przedmiot 

sort(GateKeeper.items.begin(), GateKeeper.items.end());			//Sortujemy przedmioty Straznika
sort(Player.items.begin(), Player.items.end());					//oraz gracza

//*************NIE MAMY WSZYSTKICH PRZEDMIOTOW***************
vector<string>::iterator it;									//it bedzie wskazywal na ostatni przedmiot ktorego gracz nie posiada
it=set_difference (GateKeeper.items.begin(), GateKeeper.items.end(),
				   Player.items.begin(), Player.items.end(), missingItems.begin());


cout<<"Gate Keeper says: You CANNOT pass!\nYou need items: ";									
while( b!=it)													//Wypisz przedmioty ktore gracz musi zdobyc,  								
	cout <<*(b++)<<" ";											//aby przejsc przez brame tj.
																//Silver Key, Vampire Fang
missingItems.clear();

//*************ZDOBYWAMY POTRZEBNE PRZEDMIOTY****************
Player.addItem("Silver Key");									//
Player.addItem("Vampire Fang");									//Player zdobywa potrzebne przedmioty

sort(Player.items.begin(),Player.items.end());

it=set_difference (GateKeeper.items.begin(), GateKeeper.items.end(),				//iterator nie wskazuje na zaden element,
				   Player.items.begin(), Player.items.end(), missingItems.begin()); //vector missingItems jest pusty

if(missingItems.empty())										//Straznik nas przepusci:)
	cout<<"\n\nGate Keeper says: You CAN pass!\nYou miss " << (int)(it-missingItems.begin()) << " elements\n";//0 elements
															
  return EXIT_SUCCESS;						
}